@@ -4,6 +4,8 @@ from pathlib import Path
 from uuid import uuid4
 
 import pytest
+from pydantic import HttpUrl
+
 from futurehouse_client.clients.data_storage_methods import DataStorageError
 from futurehouse_client.clients.rest_client import (
     RestClient,
@@ -15,13 +17,19 @@ from futurehouse_client.models.data_storage_methods import (
     DataContentType,
     DataStorageLocationPayload,
     DataStorageType,
+    RawFetchResponse,
+)
+from futurehouse_client.models.rest import (
+    FilterLogic,
+    SearchCriterion,
+    SearchOperator,
 )
 
 ADMIN_API_KEY = os.environ["PLAYWRIGHT_ADMIN_API_KEY"]
 
 
-@pytest.fixture
-def admin_client():
+@pytest.fixture(name="admin_client")
+def fixture_admin_client():
     """Create a RestClient for testing; using a admin user key with full access."""
     return RestClient(
         stage=Stage.DEV,
@@ -45,8 +53,10 @@ def test_store_raw_content_sync(admin_client: RestClient):
 
     fetch_response = admin_client.fetch_data_from_storage(response.data_storage.id)
 
-    assert fetch_response is not None
-    assert fetch_response == test_content
+    assert isinstance(fetch_response, RawFetchResponse)
+    assert fetch_response.content == test_content
+    assert fetch_response.entry_id == response.data_storage.id
+    assert fetch_response.entry_name == response.data_storage.name
 
     admin_client.delete_data_storage_entry(response.data_storage.id)
 
@@ -72,8 +82,10 @@ async def test_store_raw_content_async(admin_client: RestClient):
         response.data_storage.id
     )
 
-    assert fetch_response is not None
-    assert fetch_response == test_content
+    assert isinstance(fetch_response, RawFetchResponse)
+    assert fetch_response.content == test_content
+    assert fetch_response.entry_id == response.data_storage.id
+    assert fetch_response.entry_name == response.data_storage.name
 
     await admin_client.adelete_data_storage_entry(response.data_storage.id)
 
@@ -83,9 +95,10 @@ async def test_store_raw_content_async(admin_client: RestClient):
 
 @pytest.mark.timeout(300)
 def test_store_file_content_sync(admin_client: RestClient):
+    file_path = Path("packages/futurehouse-client/tests/test_data/test_file.txt")
     response = admin_client.store_file_content(
         name=f"E2E test entry file: {uuid4()}",
-        file_path=Path("packages/futurehouse-client/tests/test_data/test_file.txt"),
+        file_path=file_path,
     )
 
     assert response is not None
@@ -95,8 +108,37 @@ def test_store_file_content_sync(admin_client: RestClient):
     fetch_response = admin_client.fetch_data_from_storage(response.data_storage.id)
 
     assert fetch_response is not None
+    assert isinstance(fetch_response, RawFetchResponse)
+    assert fetch_response.filename == file_path
     assert (
-        fetch_response
+        fetch_response.content
+        == "Here is some random text that shall immortalize Eddie's brain in code.\n"
+    )
+
+    admin_client.delete_data_storage_entry(response.data_storage.id)
+
+
+@pytest.mark.timeout(300)
+def test_store_file_content_with_path_override_sync(admin_client: RestClient):
+    file_path = Path("packages/futurehouse-client/tests/test_data/test_file.txt")
+    file_path_override = Path("test_file_override.txt")
+    response = admin_client.store_file_content(
+        name=f"E2E test entry file: {uuid4()}",
+        file_path=file_path,
+        file_path_override=file_path_override,
+    )
+
+    assert response is not None
+    assert response.data_storage.id is not None
+    assert len(response.storage_locations) > 0
+
+    fetch_response = admin_client.fetch_data_from_storage(response.data_storage.id)
+
+    assert fetch_response is not None
+    assert isinstance(fetch_response, RawFetchResponse)
+    assert fetch_response.filename == file_path_override
+    assert (
+        fetch_response.content
         == "Here is some random text that shall immortalize Eddie's brain in code.\n"
     )
 
@@ -126,9 +168,10 @@ def test_store_dir_content_sync(admin_client: RestClient):
 @pytest.mark.timeout(300)
 @pytest.mark.asyncio
 async def test_store_file_content_async(admin_client: RestClient):
+    file_path = Path("packages/futurehouse-client/tests/test_data/test_file.txt")
     response = await admin_client.astore_file_content(
         name=f"E2E test entry file: {uuid4()}",
-        file_path=Path("packages/futurehouse-client/tests/test_data/test_file.txt"),
+        file_path=file_path,
     )
 
     assert response is not None
@@ -140,8 +183,40 @@ async def test_store_file_content_async(admin_client: RestClient):
     )
 
     assert fetch_response is not None
+    assert isinstance(fetch_response, RawFetchResponse)
+    assert fetch_response.filename == file_path
     assert (
-        fetch_response
+        fetch_response.content
+        == "Here is some random text that shall immortalize Eddie's brain in code.\n"
+    )
+
+    await admin_client.adelete_data_storage_entry(response.data_storage.id)
+
+
+@pytest.mark.timeout(300)
+@pytest.mark.asyncio
+async def test_store_file_content_with_path_override_async(admin_client: RestClient):
+    file_path = Path("packages/futurehouse-client/tests/test_data/test_file.txt")
+    file_path_override = Path("test_file_override.txt")
+    response = await admin_client.astore_file_content(
+        name=f"E2E test entry file: {uuid4()}",
+        file_path=file_path,
+        file_path_override=file_path_override,
+    )
+
+    assert response is not None
+    assert response.data_storage.id is not None
+    assert len(response.storage_locations) > 0
+
+    fetch_response = await admin_client.afetch_data_from_storage(
+        response.data_storage.id
+    )
+
+    assert fetch_response is not None
+    assert isinstance(fetch_response, RawFetchResponse)
+    assert fetch_response.filename == file_path_override
+    assert (
+        fetch_response.content
         == "Here is some random text that shall immortalize Eddie's brain in code.\n"
     )
 
@@ -250,80 +325,82 @@ async def test_register_existing_content_gcs_async_collection(admin_client: Rest
     await admin_client.adelete_data_storage_entry(response.data_storage.id)
 
 
-# @pytest.mark.timeout(300)
-# @pytest.mark.asyncio
-# async def test_register_existing_content_gcs_async_single(admin_client: RestClient):
-#     response = await admin_client.aregister_existing_data_source(
-#         name=f"E2E test entry gcs dir: {uuid4()}",
-#         description="This is data that already exists",
-#         as_collection=False,
-#         existing_location=DataStorageLocationPayload(
-#             storage_type=DataStorageType.GCS,
-#             content_type=DataContentType.DIRECTORY,
-#             metadata={"bucket_name": "fh-pubmed-data"},
-#             location="oa_package/00/00/PMC10054724.tar.gz",
-#         ),
-#     )
+@pytest.mark.timeout(300)
+@pytest.mark.asyncio
+async def test_register_existing_content_gcs_async_single(admin_client: RestClient):
+    response = await admin_client.aregister_existing_data_source(
+        name=f"E2E test entry gcs dir: {uuid4()}",
+        description="This is data that already exists",
+        as_collection=False,
+        existing_location=DataStorageLocationPayload(
+            storage_type=DataStorageType.GCS,
+            content_type=DataContentType.DIRECTORY,
+            metadata={"bucket_name": "fh-pubmed-data"},
+            location="oa_package/00/00/PMC10054724.tar.gz",
+        ),
+    )
 
-#     assert response is not None
-#     assert response.data_storage.id is not None
-#     assert len(response.storage_locations) > 0
+    assert response is not None
+    assert response.data_storage.id is not None
+    assert len(response.storage_locations) > 0
 
-#     fetch_response = await admin_client.afetch_data_from_storage(
-#         response.data_storage.id
-#     )
+    fetch_response = await admin_client.afetch_data_from_storage(
+        response.data_storage.id
+    )
 
-#     assert isinstance(fetch_response, Path)
-#     assert fetch_response.exists()
+    assert isinstance(fetch_response, Path)
+    assert fetch_response.exists()
+    assert fetch_response.name == "PMC10054724.tar.gz"
 
 
-# @pytest.mark.timeout(300)
-# def test_register_existing_content_gcs_sync_single(admin_client: RestClient):
-#     response = admin_client.register_existing_data_source(
-#         name=f"E2E test entry gcs dir: {uuid4()}",
-#         description="This is data that already exists",
-#         as_collection=False,
-#         existing_location=DataStorageLocationPayload(
-#             storage_type=DataStorageType.GCS,
-#             content_type=DataContentType.DIRECTORY,
-#             metadata={"bucket_name": "fh-pubmed-data"},
-#             location="oa_package/00/00/PMC10054724.tar.gz",
-#         ),
-#     )
+@pytest.mark.timeout(300)
+def test_register_existing_content_gcs_sync_single(admin_client: RestClient):
+    response = admin_client.register_existing_data_source(
+        name=f"E2E test entry gcs dir: {uuid4()}",
+        description="This is data that already exists",
+        as_collection=False,
+        existing_location=DataStorageLocationPayload(
+            storage_type=DataStorageType.GCS,
+            content_type=DataContentType.DIRECTORY,
+            metadata={"bucket_name": "fh-pubmed-data"},
+            location="oa_package/00/00/PMC10054724.tar.gz",
+        ),
+    )
 
-#     assert response is not None
-#     assert response.data_storage.id is not None
-#     assert len(response.storage_locations) > 0
+    assert response is not None
+    assert response.data_storage.id is not None
+    assert len(response.storage_locations) > 0
 
-#     fetch_response = admin_client.fetch_data_from_storage(response.data_storage.id)
+    fetch_response = admin_client.fetch_data_from_storage(response.data_storage.id)
 
-#     assert isinstance(fetch_response, Path)
-#     assert fetch_response.exists()
+    assert isinstance(fetch_response, Path)
+    assert fetch_response.exists()
+    assert fetch_response.name == "PMC10054724.tar.gz"
 
 
-# @pytest.mark.timeout(300)
-# @pytest.mark.asyncio
-# async def test_register_existing_content_gcs_async_single_with_prefix(
-#     admin_client: RestClient,
-# ):
-#     with pytest.raises(
-#         DataStorageError,
-#         match="Prefix is not allowed for single file GCS storage",
-#     ):
-#         await admin_client.aregister_existing_data_source(
-#             name=f"E2E test entry gcs dir: {uuid4()}",
-#             description="This is data that already exists",
-#             as_collection=False,
-#             existing_location=DataStorageLocationPayload(
-#                 storage_type=DataStorageType.GCS,
-#                 content_type=DataContentType.DIRECTORY,
-#                 metadata={
-#                     "bucket_name": "fh-pubmed-data",
-#                     "prefix": "oa_package/00/00",
-#                 },
-#                 location="oa_package/00/00/PMC10054724.tar.gz",
-#             ),
-#         )
+@pytest.mark.timeout(300)
+@pytest.mark.asyncio
+async def test_register_existing_content_gcs_async_single_with_prefix(
+    admin_client: RestClient,
+):
+    with pytest.raises(
+        DataStorageError,
+        match="Prefix is not allowed for single file GCS storage",
+    ):
+        await admin_client.aregister_existing_data_source(
+            name=f"E2E test entry gcs dir: {uuid4()}",
+            description="This is data that already exists",
+            as_collection=False,
+            existing_location=DataStorageLocationPayload(
+                storage_type=DataStorageType.GCS,
+                content_type=DataContentType.DIRECTORY,
+                metadata={
+                    "bucket_name": "fh-pubmed-data",
+                    "prefix": "oa_package/00/00",
+                },
+                location="oa_package/00/00/PMC10054724.tar.gz",
+            ),
+        )
 
 
 @pytest.mark.timeout(300)
@@ -349,8 +426,8 @@ def test_register_existing_content_postgres_sync(admin_client: RestClient):
 
     fetch_response = admin_client.fetch_data_from_storage(response.data_storage.id)
 
-    assert isinstance(fetch_response, str)
-    assert json.loads(fetch_response).get("id") == test_trajectory_id
+    assert isinstance(fetch_response, RawFetchResponse)
+    assert json.loads(fetch_response.content).get("id") == test_trajectory_id
 
     admin_client.delete_data_storage_entry(response.data_storage.id)
 
@@ -381,8 +458,8 @@ async def test_register_existing_content_postgres_async(admin_client: RestClient
         response.data_storage.id
     )
 
-    assert isinstance(fetch_response, str)
-    assert json.loads(fetch_response).get("id") == test_trajectory_id
+    assert isinstance(fetch_response, RawFetchResponse)
+    assert json.loads(fetch_response.content).get("id") == test_trajectory_id
 
     await admin_client.adelete_data_storage_entry(response.data_storage.id)
 
@@ -434,51 +511,164 @@ async def test_register_existing_content_bigquery_async(admin_client: RestClient
     await admin_client.adelete_data_storage_entry(response.data_storage.id)
 
 
+class TestDataset:
+    @pytest.mark.timeout(300)
+    @pytest.mark.asyncio
+    async def test_dataset_async(self, admin_client: RestClient):
+        dataset_name_id = uuid4()
+        create_response = await admin_client.acreate_dataset(
+            name=f"E2E test dataset: {dataset_name_id}",
+            description="This is a test dataset",
+        )
+
+        assert create_response is not None
+        assert create_response.id is not None
+        assert create_response.name == f"E2E test dataset: {dataset_name_id}"
+        assert create_response.description == "This is a test dataset"
+
+        get_response = await admin_client.aget_dataset(dataset_id=create_response.id)
+
+        assert get_response is not None
+        assert get_response.dataset.id == create_response.id
+        assert get_response.data_storage_entries == []
+
+        await admin_client.adelete_dataset(dataset_id=create_response.id)
+
+        with pytest.raises(DataStorageError, match="Failed to get dataset"):
+            await admin_client.aget_dataset(dataset_id=create_response.id)
+
+    @pytest.mark.timeout(300)
+    def test_dataset_sync(self, admin_client: RestClient):
+        dataset_name_id = uuid4()
+        create_response = admin_client.create_dataset(
+            name=f"E2E test dataset: {dataset_name_id}",
+            description="This is a test dataset",
+        )
+
+        assert create_response is not None
+        assert create_response.id is not None
+        assert create_response.name == f"E2E test dataset: {dataset_name_id}"
+
+        get_response = admin_client.get_dataset(dataset_id=create_response.id)
+
+        assert get_response is not None
+        assert get_response.dataset.id == create_response.id
+        assert get_response.data_storage_entries == []
+
+        admin_client.delete_dataset(dataset_id=create_response.id)
+
+        with pytest.raises(DataStorageError, match="Failed to get dataset"):
+            admin_client.get_dataset(dataset_id=create_response.id)
+
+
 @pytest.mark.timeout(300)
 @pytest.mark.asyncio
-async def test_dataset_async(admin_client: RestClient):
-    dataset_name_id = uuid4()
-    create_response = await admin_client.acreate_dataset(
-        name=f"E2E test dataset: {dataset_name_id}",
-        description="This is a test dataset",
+async def test_astore_link_success(admin_client: RestClient):
+    """Test successful asynchronous link storage."""
+    result = await admin_client.astore_link(
+        name="Test API Link",
+        url=HttpUrl("https://xyz.api.futurehouse.org"),
+        description="Test link to xyz.api.futurehouse.org",
+        instructions="this is a test api which you can call without authentication",
+        api_key="test_key_123",
     )
 
-    assert create_response is not None
-    assert create_response.id is not None
-    assert create_response.name == f"E2E test dataset: {dataset_name_id}"
-    assert create_response.description == "This is a test dataset"
-
-    get_response = await admin_client.aget_dataset(dataset_id=create_response.id)
-
-    assert get_response is not None
-    assert get_response["dataset"]["id"] == str(create_response.id)
-    assert get_response["data_storage_entries"] == []
-
-    await admin_client.adelete_dataset(dataset_id=create_response.id)
-
-    with pytest.raises(DataStorageError, match="Failed to get dataset"):
-        await admin_client.aget_dataset(dataset_id=create_response.id)
-
-
-@pytest.mark.timeout(300)
-def test_dataset_sync(admin_client: RestClient):
-    dataset_name_id = uuid4()
-    create_response = admin_client.create_dataset(
-        name=f"E2E test dataset: {dataset_name_id}",
-        description="This is a test dataset",
+    assert result.data_storage.id is not None
+    assert result.data_storage.name == "Test API Link"
+    assert result.data_storage.description == "Test link to xyz.api.futurehouse.org"
+    assert result.data_storage.content == "https://xyz.api.futurehouse.org/"
+    assert len(result.storage_locations) > 0
+    assert result.storage_locations[0].storage_config.storage_type == "link"
+    assert (
+        result.storage_locations[0].storage_config.location
+        == "https://xyz.api.futurehouse.org/"
     )
 
-    assert create_response is not None
-    assert create_response.id is not None
-    assert create_response.name == f"E2E test dataset: {dataset_name_id}"
 
-    get_response = admin_client.get_dataset(dataset_id=create_response.id)
+class TestDataStorageSearch:
+    @pytest.mark.timeout(300)
+    def test_data_storage_search_sync(self, admin_client: RestClient):
+        test_query = "test"
 
-    assert get_response is not None
-    assert get_response["dataset"]["id"] == str(create_response.id)
-    assert get_response["data_storage_entries"] == []
+        test_criteria = SearchCriterion(
+            field="name", operator=SearchOperator.CONTAINS, value=test_query
+        )
 
-    admin_client.delete_dataset(dataset_id=create_response.id)
+        results = admin_client.search_data_storage(criteria=[test_criteria], size=5)
 
-    with pytest.raises(DataStorageError, match="Failed to get dataset"):
-        admin_client.get_dataset(dataset_id=create_response.id)
+        assert results is not None
+        assert len(results) == 5
+        assert test_query.lower() in results[0]["name"].lower()
+
+    @pytest.mark.timeout(300)
+    @pytest.mark.asyncio
+    async def test_data_storage_search_async(self, admin_client: RestClient):
+        test_query = "test"
+
+        name_test_criteria = SearchCriterion(
+            field="name", operator=SearchOperator.CONTAINS, value=test_query
+        )
+
+        results = await admin_client.asearch_data_storage(
+            criteria=[name_test_criteria], size=5
+        )
+
+        assert results is not None
+        assert len(results) == 5
+        assert test_query.lower() in results[0]["name"].lower()
+
+    @pytest.mark.timeout(300)
+    @pytest.mark.asyncio
+    async def test_complex_data_storage_search_async(self, admin_client: RestClient):
+        test_query = "test"
+
+        name_test_criteria = SearchCriterion(
+            field="name", operator=SearchOperator.CONTAINS, value=test_query
+        )
+
+        user_id_test_criteria = SearchCriterion(
+            field="user_id",
+            operator=SearchOperator.EQUALS,
+            value="a5E0sDbOYEWLBoy9qzOQ8yfKiKw1",
+        )
+
+        dataset_id_test_criteria = SearchCriterion(
+            field="dataset_id",
+            operator=SearchOperator.EQUALS,
+            value="8bb27d71-8ec6-4845-a09e-4eceefb38cba",
+        )
+
+        description_test_criteria = SearchCriterion(
+            field="description",
+            operator=SearchOperator.CONTAINS,
+            value="xyz.api.futurehouse.org",
+        )
+
+        or_results = await admin_client.asearch_data_storage(
+            criteria=[
+                name_test_criteria,
+                user_id_test_criteria,
+                dataset_id_test_criteria,
+                description_test_criteria,
+            ],
+            filter_logic=FilterLogic.OR,
+        )
+
+        and_results = await admin_client.asearch_data_storage(
+            criteria=[
+                name_test_criteria,
+                user_id_test_criteria,
+                dataset_id_test_criteria,
+                description_test_criteria,
+            ],
+            filter_logic=FilterLogic.AND,
+        )
+
+        assert or_results is not None
+        assert len(or_results) > 0
+
+        assert and_results is not None
+        assert len(and_results) > 0
+        assert test_query.lower() in and_results[0]["name"].lower()
+
+        assert len(or_results) > len(and_results)
