@@ -7,6 +7,8 @@ from unittest.mock import ANY, AsyncMock, Mock, patch
 from uuid import uuid4
 
 import pytest
+from httpx import HTTPStatusError, codes
+
 from futurehouse_client.clients.data_storage_methods import (
     DataStorageCreationError,
     DataStorageError,
@@ -20,8 +22,8 @@ from futurehouse_client.models.data_storage_methods import (
     DataStorageResponse,
     DirectoryManifest,
     ManifestEntry,
+    RawFetchResponse,
 )
-from httpx import HTTPStatusError, codes
 
 
 class MockDataStorageMethods(DataStorageMethods):
@@ -70,6 +72,7 @@ class TestDataStorageMethods:
                 "created_at": now,
                 "modified_at": now,
                 "content": content,
+                "status": "active",
             },
             "storage_locations": [
                 {
@@ -484,7 +487,7 @@ class TestDataStorageMethods:
                 json={
                     "name": "test_dir",
                     "description": "Test directory",
-                    "path": "test_dir.zip",  # The actual path includes .zip extension
+                    "file_path": "test_dir.zip",  # The actual path includes .zip extension
                     "is_collection": True,
                 },
             )
@@ -531,7 +534,7 @@ class TestDataStorageMethods:
                 json={
                     "name": "test_dir",
                     "description": "Test directory",
-                    "path": "test_dir.zip",  # The actual path includes .zip extension
+                    "file_path": "test_dir.zip",  # The actual path includes .zip extension
                     "is_collection": True,
                 },
             )
@@ -783,10 +786,10 @@ class TestDataStorageMethods:
         result = self.methods.fetch_data_from_storage(self.test_uuid)
 
         # Verify raw content storage returns the content directly
-        assert isinstance(result, str)
-        assert result == "test content"
-        assert len(result) == 12  # "test content" is 12 characters
-        assert result.strip() == result  # No leading/trailing whitespace
+        assert isinstance(result, RawFetchResponse)
+        assert result.content == "test content"
+        assert result.entry_id == self.test_uuid
+        assert result.entry_name == "test"
 
     def test_fetch_data_from_storage_no_id(self):
         """Test fetching data without providing storage ID."""
@@ -1045,12 +1048,34 @@ class TestDataStorageMethods:
         """Test successful asynchronous dataset retrieval."""
         mock_response = Mock()
         mock_response.json.return_value = {
-            "id": str(self.test_uuid),
-            "name": "test_dataset",
-            "description": "Test dataset description",
-            "user_id": "test_user_123",
-            "created_at": "2024-01-01T00:00:00Z",
-            "modified_at": "2024-01-01T00:00:00Z",
+            "dataset": {
+                "id": str(self.test_uuid),
+                "name": "test_dataset",
+                "description": "Test dataset description",
+                "user_id": "test_user_123",
+                "created_at": "2024-01-01T00:00:00Z",
+                "modified_at": "2024-01-01T00:00:00Z",
+            },
+            "data_storage_entries": [
+                {
+                    "id": str(uuid4()),
+                    "name": "test_entry",
+                    "description": "Test entry description",
+                    "content": "Test content",
+                    "embedding": None,
+                    "is_collection": False,
+                    "status": "active",
+                    "tags": None,
+                    "parent_id": None,
+                    "project_id": str(self.test_project_id),
+                    "dataset_id": str(self.test_uuid),
+                    "path": None,
+                    "bigquery_schema": None,
+                    "user_id": "test_user_123",
+                    "created_at": "2024-01-01T00:00:00Z",
+                    "modified_at": "2024-01-01T00:00:00Z",
+                }
+            ],
         }
         mock_response.raise_for_status.return_value = None
 
@@ -1062,10 +1087,17 @@ class TestDataStorageMethods:
 
         # Verify the result structure and content
         assert result is not None
-        assert result["id"] == str(self.test_uuid)
-        assert result["name"] == "test_dataset"
-        assert result["description"] == "Test dataset description"
-        assert result["user_id"] == "test_user_123"
+        assert result.dataset.id == self.test_uuid
+        assert result.dataset.name == "test_dataset"
+        assert result.dataset.description == "Test dataset description"
+        assert result.dataset.user_id == "test_user_123"
+
+        # Verify data storage entries
+        assert len(result.data_storage_entries) == 1
+        assert result.data_storage_entries[0].name == "test_entry"
+        assert result.data_storage_entries[0].description == "Test entry description"
+        assert result.data_storage_entries[0].content == "Test content"
+        assert result.data_storage_entries[0].dataset_id == self.test_uuid
 
         # Verify the API call was made with the correct endpoint
         self.methods.async_client.get.assert_called_once_with(
@@ -1181,28 +1213,6 @@ class TestDataStorageMethods:
                 description="Test description",
             )
 
-    @pytest.mark.asyncio
-    async def test_aget_dataset_empty_response(self):
-        """Test dataset retrieval with empty response."""
-        mock_response = Mock()
-        mock_response.json.return_value = {}
-        mock_response.raise_for_status.return_value = None
-
-        # Make the get method return the mock response asynchronously
-        self.methods.async_client.get = AsyncMock(return_value=mock_response)
-
-        dataset_id = uuid4()
-        result = await self.methods.aget_dataset(dataset_id)
-
-        # Verify the result is an empty dict
-        assert result == {}
-        assert len(result) == 0
-
-        # Verify the API call was made with the correct endpoint
-        self.methods.async_client.get.assert_called_once_with(
-            f"/v0.1/data-storage/datasets/{dataset_id}"
-        )
-
     @pytest.mark.asyncio
     async def test_adelete_dataset_not_found(self):
         """Test dataset deletion when dataset doesn't exist."""
@@ -1231,6 +1241,122 @@ class TestDataStorageMethods:
         with pytest.raises(DataStorageError, match="Data storage entry not found"):
             await self.methods.adelete_data_storage_entry(entry_id)
 
+    def test_get_dataset_success(self):
+        """Test successful synchronous dataset retrieval."""
+        mock_response = Mock()
+        mock_response.json.return_value = {
+            "dataset": {
+                "id": str(self.test_uuid),
+                "name": "test_dataset",
+                "description": "Test dataset description",
+                "user_id": "test_user_123",
+                "created_at": "2024-01-01T00:00:00Z",
+                "modified_at": "2024-01-01T00:00:00Z",
+            },
+            "data_storage_entries": [
+                {
+                    "id": str(uuid4()),
+                    "name": "test_entry",
+                    "description": "Test entry description",
+                    "content": "Test content",
+                    "embedding": None,
+                    "is_collection": False,
+                    "status": "active",
+                    "tags": None,
+                    "parent_id": None,
+                    "project_id": str(self.test_project_id),
+                    "dataset_id": str(self.test_uuid),
+                    "path": None,
+                    "bigquery_schema": None,
+                    "user_id": "test_user_123",
+                    "created_at": "2024-01-01T00:00:00Z",
+                    "modified_at": "2024-01-01T00:00:00Z",
+                }
+            ],
+        }
+        mock_response.raise_for_status.return_value = None
+
+        # Make the get method return the mock response
+        self.methods.client.get = Mock(return_value=mock_response)
+
+        dataset_id = uuid4()
+        result = self.methods.get_dataset(dataset_id)
+
+        # Verify the result structure and content
+        assert result is not None
+        assert result.dataset.id == self.test_uuid
+        assert result.dataset.name == "test_dataset"
+        assert result.dataset.description == "Test dataset description"
+        assert result.dataset.user_id == "test_user_123"
+
+        # Verify data storage entries
+        assert len(result.data_storage_entries) == 1
+        assert result.data_storage_entries[0].name == "test_entry"
+        assert result.data_storage_entries[0].description == "Test entry description"
+        assert result.data_storage_entries[0].content == "Test content"
+        assert result.data_storage_entries[0].dataset_id == self.test_uuid
+
+        # Verify the API call was made with the correct endpoint
+        self.methods.client.get.assert_called_once_with(
+            f"/v0.1/data-storage/datasets/{dataset_id}"
+        )
+
+    def test_get_dataset_http_error(self):
+        """Test synchronous dataset retrieval with HTTP error."""
+        mock_response = Mock()
+        mock_response.status_code = 404
+        mock_response.text = "Dataset not found"
+
+        error = HTTPStatusError("Not Found", request=Mock(), response=mock_response)
+        self.methods.client.get = Mock(side_effect=error)
+
+        dataset_id = uuid4()
+        with pytest.raises(DataStorageError, match="Dataset not found"):
+            self.methods.get_dataset(dataset_id)
+
+    def test_get_dataset_unexpected_error(self):
+        """Test synchronous dataset retrieval with unexpected error."""
+        unexpected_error = ValueError("Unexpected error")
+        self.methods.client.get = Mock(side_effect=unexpected_error)
+
+        dataset_id = uuid4()
+        with pytest.raises(DataStorageError, match="An unexpected error occurred"):
+            self.methods.get_dataset(dataset_id)
+
+    def test_get_dataset_empty_response(self):
+        """Test synchronous dataset retrieval with minimal response."""
+        mock_response = Mock()
+        mock_response.json.return_value = {
+            "dataset": {
+                "id": str(uuid4()),
+                "name": "",
+                "user_id": "test_user_123",
+                "created_at": "2024-01-01T00:00:00Z",
+                "modified_at": "2024-01-01T00:00:00Z",
+            },
+            "data_storage_entries": [],
+        }
+        mock_response.raise_for_status.return_value = None
+
+        # Make the get method return the mock response
+        self.methods.client.get = Mock(return_value=mock_response)
+
+        dataset_id = uuid4()
+        result = self.methods.get_dataset(dataset_id)
+
+        # Verify the result structure
+        assert result is not None
+        assert result.dataset.id is not None
+        assert not result.dataset.name
+        assert result.dataset.user_id == "test_user_123"
+        assert result.data_storage_entries == []
+        assert len(result.data_storage_entries) == 0
+
+        # Verify the API call was made with the correct endpoint
+        self.methods.client.get.assert_called_once_with(
+            f"/v0.1/data-storage/datasets/{dataset_id}"
+        )
+
 
 class TestUtilityFunctions:
     """Test cases for utility functions."""
