@@ -1,6 +1,6 @@
 # ruff: noqa: ARG001
-# ruff:  noqa: SIM117
-# pylint: disable=too-many-lines,import-error,too-many-public-methods
+# ruff: noqa: SIM117
+
 import asyncio
 import os
 import tempfile
@@ -12,32 +12,6 @@ from unittest.mock import MagicMock, Mock, mock_open, patch
 from uuid import UUID, uuid4
 
 import pytest
-from futurehouse_client.clients import (
-    JobNames,
-)
-from futurehouse_client.clients.rest_client import (
-    FileUploadError,
-    ProjectError,
-    RestClient,
-    RestClientError,
-    UserAgentRequestCreationError,
-    UserAgentRequestFetchError,
-)
-from futurehouse_client.models.app import (
-    PhoenixTaskResponse,
-    PQATaskResponse,
-    Stage,
-    TaskRequest,
-    TaskResponseVerbose,
-)
-from futurehouse_client.models.rest import (
-    ExecutionStatus,
-    UserAgentRequestPostPayload,
-    UserAgentRequestStatus,
-    UserAgentResponsePayload,
-    WorldModel,
-)
-from futurehouse_client.utils.general import create_retry_if_connection_error
 from httpx import (
     CloseError,
     ConnectError,
@@ -51,13 +25,50 @@ from httpx import (
 from pytest_subtests import SubTests
 from requests.exceptions import RequestException, Timeout
 
+from futurehouse_client.clients import (
+    JobNames,
+)
+from futurehouse_client.clients.rest_client import (
+    FileUploadError,
+    JobEventCreationError,
+    JobEventUpdateError,
+    ProjectError,
+    RestClient,
+    RestClientError,
+    UserAgentRequestCreationError,
+    UserAgentRequestFetchError,
+)
+from futurehouse_client.models.app import (
+    LiteTaskResponse,
+    PhoenixTaskResponse,
+    PQATaskResponse,
+    Stage,
+    TaskRequest,
+    TaskResponse,
+    TaskResponseVerbose,
+)
+from futurehouse_client.models.job_event import (
+    CostComponent,
+    ExecutionType,
+    JobEventCreateRequest,
+    JobEventUpdateRequest,
+)
+from futurehouse_client.models.rest import (
+    ExecutionStatus,
+    UserAgentRequestPostPayload,
+    UserAgentRequestStatus,
+    UserAgentResponsePayload,
+    WorldModel,
+)
+from futurehouse_client.utils.general import create_retry_if_connection_error
+
 ADMIN_API_KEY = os.environ.get("PLAYWRIGHT_ADMIN_API_KEY", "")
 PUBLIC_API_KEY = os.environ.get("PLAYWRIGHT_PUBLIC_API_KEY", "")
 TEST_MAX_POLLS = 100
 
 
-@pytest.fixture
-def admin_client():
+@pytest.fixture(name="admin_client")
+def fixture_admin_client():
     """Create a RestClient for testing; using an admin key."""
     return RestClient(
         stage=Stage.DEV,
@@ -65,8 +76,8 @@ def admin_client():
     )
 
 
-@pytest.fixture
-def pub_client():
+@pytest.fixture(name="pub_client")
+def fixture_pub_client():
     """Create a RestClient for testing; using a public user key with limited access."""
     return RestClient(
         stage=Stage.DEV,
@@ -74,8 +85,8 @@ def pub_client():
     )
 
 
-@pytest.fixture
-def task_req():
+@pytest.fixture(name="task_req")
+def fixture_task_req():
     """Create a sample task request."""
     return TaskRequest(
         name=JobNames.from_string("dummy"),
@@ -83,24 +94,26 @@ def task_req():
     )
 
 
-@pytest.fixture
-def pqa_task_req():
+@pytest.fixture(name="pqa_task_req")
+def fixture_pqa_task_req():
     return TaskRequest(
         name=JobNames.from_string("crow"),
         query="How many moons does earth have?",
     )
 
 
-@pytest.fixture
-def phoenix_task_req():
+@pytest.fixture(name="phoenix_task_req")
+def fixture_phoenix_task_req():
     return TaskRequest(
         name=JobNames.from_string("phoenix"),
         query="What is the molecular weight of ascorbic acids?",
     )
 
 
-@pytest.fixture
-def running_trajectory_id(admin_client: RestClient, task_req: TaskRequest) -> str:
+@pytest.fixture(name="running_trajectory_id")
+def fixture_running_trajectory_id(
+    admin_client: RestClient, task_req: TaskRequest
+) -> str:
     return admin_client.create_task(task_req)
 
 
@@ -139,10 +152,12 @@ async def test_job_response(  # noqa: PLR0915
     with subtests.test("Test TaskResponse with queued task"):
         task_response = admin_client.get_task(task_id)
         assert task_response.status in {"queued", "in progress"}
+        assert not isinstance(task_response, LiteTaskResponse)
         assert task_response.job_name == pqa_task_req.name
         assert task_response.query == pqa_task_req.query
         task_response = await admin_client.aget_task(atask_id)
         assert task_response.status in {"queued", "in progress"}
+        assert not isinstance(task_response, LiteTaskResponse)
         assert task_response.job_name == pqa_task_req.name
         assert task_response.query == pqa_task_req.query
 
@@ -230,6 +245,25 @@ def test_run_until_done_futurehouse_dummy_env_crow(
     assert all(task.status == "success" for task in results)
 
 
+@pytest.mark.timeout(300)
+@pytest.mark.flaky(reruns=3)
+def test_run_until_done_returns_task_response(
+    admin_client: RestClient, task_req: TaskRequest
+):
+    """Test that run_tasks_until_done returns TaskResponse instead of LiteTaskResponse."""
+    tasks_to_do = [task_req, task_req]
+
+    results = admin_client.run_tasks_until_done(tasks_to_do)
+
+    assert len(results) == len(tasks_to_do), "Should return 2 tasks."
+    assert all(task.status == "success" for task in results)
+
+    for task in results:
+        assert isinstance(task, TaskResponse), (
+            f"Expected TaskResponse, got {type(task)}"
+        )
+
+
 @pytest.mark.timeout(300)
 @pytest.mark.flaky(reruns=3)
 @pytest.mark.asyncio
@@ -244,6 +278,26 @@ async def test_arun_until_done_futurehouse_dummy_env_crow(
     assert all(task.status == "success" for task in results)
 
 
+@pytest.mark.timeout(300)
+@pytest.mark.flaky(reruns=3)
+@pytest.mark.asyncio
+async def test_arun_until_done_returns_task_response(
+    admin_client: RestClient, task_req: TaskRequest
+):
+    """Test that arun_tasks_until_done returns TaskResponse instead of LiteTaskResponse."""
+    tasks_to_do = [task_req, task_req]
+
+    results = await admin_client.arun_tasks_until_done(tasks_to_do)
+
+    assert len(results) == len(tasks_to_do), "Should return 2 tasks."
+    assert all(task.status == "success" for task in results)
+
+    for task in results:
+        assert isinstance(task, TaskResponse), (
+            f"Expected TaskResponse, got {type(task)}"
+        )
+
+
 @pytest.mark.timeout(300)
 @pytest.mark.flaky(reruns=3)
 @pytest.mark.asyncio
@@ -1226,3 +1280,144 @@ def test_retry_logic_conditions(exception, should_retry, test_description):
     assert result == should_retry, (
         f"Expected {should_retry} for {test_description}, got {result}"
     )
+
+
+class TestJobEventOperations:
+    @pytest.fixture
+    def test_trajectory_id(
+        self, admin_client: RestClient, task_req: TaskRequest
+    ) -> str:
+        """Create a real trajectory for job event testing."""
+        return admin_client.create_task(task_req)
+
+    @pytest.fixture
+    def job_event_create_request(self, test_trajectory_id: str):
+        from datetime import datetime
+
+        return JobEventCreateRequest(
+            execution_id=UUID(test_trajectory_id),
+            execution_type=ExecutionType.TRAJECTORY,
+            cost_component=CostComponent.LLM_USAGE,
+            started_at=datetime.now(),
+            ended_at=datetime.now(),
+            crow=None,
+            amount_usd=0.005,
+            rate=0.0001,
+            input_token_count=100,
+            completion_token_count=50,
+            metadata={"model": "gpt-4o", "temperature": 0.7},
+        )
+
+    @pytest.fixture
+    def job_event_update_request(self):
+        return JobEventUpdateRequest(
+            amount_usd=0.007,
+            rate=0.00015,
+            input_token_count=120,
+            completion_token_count=60,
+            metadata={"model": "gpt-4o", "temperature": 0.5, "updated": True},
+        )
+
+    @pytest.mark.timeout(300)
+    def test_create_job_event_success(
+        self, admin_client: RestClient, job_event_create_request
+    ):
+        response = admin_client.create_job_event(job_event_create_request)
+        assert response.id is not None
+        assert isinstance(response.id, UUID)
+
+    @pytest.mark.asyncio
+    @pytest.mark.timeout(300)
+    async def test_acreate_job_event_success(
+        self, admin_client: RestClient, job_event_create_request
+    ):
+        response = await admin_client.acreate_job_event(job_event_create_request)
+        assert response.id is not None
+        assert isinstance(response.id, UUID)
+
+    def test_create_job_event_execution_not_found(
+        self, admin_client: RestClient, job_event_create_request
+    ):
+        # Create a new request with a non-existent execution ID
+        invalid_request = JobEventCreateRequest(
+            execution_id=uuid4(),  # Non-existent ID
+            execution_type=job_event_create_request.execution_type,
+            cost_component=job_event_create_request.cost_component,
+            started_at=job_event_create_request.started_at,
+            ended_at=job_event_create_request.ended_at,
+            crow=job_event_create_request.crow,
+            amount_usd=job_event_create_request.amount_usd,
+            rate=job_event_create_request.rate,
+            input_token_count=job_event_create_request.input_token_count,
+            completion_token_count=job_event_create_request.completion_token_count,
+            metadata=job_event_create_request.metadata,
+        )
+        with pytest.raises(JobEventCreationError, match="Execution not found"):
+            admin_client.create_job_event(invalid_request)
+
+    @pytest.mark.asyncio
+    async def test_acreate_job_event_execution_not_found(
+        self, admin_client: RestClient, job_event_create_request
+    ):
+        # Create a new request with a non-existent execution ID
+        invalid_request = JobEventCreateRequest(
+            execution_id=uuid4(),  # Non-existent ID
+            execution_type=job_event_create_request.execution_type,
+            cost_component=job_event_create_request.cost_component,
+            started_at=job_event_create_request.started_at,
+            ended_at=job_event_create_request.ended_at,
+            crow=job_event_create_request.crow,
+            amount_usd=job_event_create_request.amount_usd,
+            rate=job_event_create_request.rate,
+            input_token_count=job_event_create_request.input_token_count,
+            completion_token_count=job_event_create_request.completion_token_count,
+            metadata=job_event_create_request.metadata,
+        )
+        with pytest.raises(JobEventCreationError, match="Execution not found"):
+            await admin_client.acreate_job_event(invalid_request)
+
+    @pytest.mark.timeout(300)
+    def test_update_job_event_success(
+        self,
+        admin_client: RestClient,
+        job_event_create_request,
+        job_event_update_request,
+    ):
+        create_response = admin_client.create_job_event(job_event_create_request)
+        job_event_id = create_response.id
+
+        # Should not raise an exception and return None
+        result = admin_client.update_job_event(job_event_id, job_event_update_request)
+        assert result is None
+
+    @pytest.mark.asyncio
+    @pytest.mark.timeout(300)
+    async def test_aupdate_job_event_success(
+        self,
+        admin_client: RestClient,
+        job_event_create_request,
+        job_event_update_request,
+    ):
+        create_response = await admin_client.acreate_job_event(job_event_create_request)
+        job_event_id = create_response.id
+
+        await admin_client.aupdate_job_event(job_event_id, job_event_update_request)
+
+    def test_update_job_event_not_found(
+        self, admin_client: RestClient, job_event_update_request
+    ):
+        job_event_id = uuid4()
+        with pytest.raises(
+            JobEventUpdateError, match=r"Job event with ID .* not found"
+        ):
+            admin_client.update_job_event(job_event_id, job_event_update_request)
+
+    @pytest.mark.asyncio
+    async def test_aupdate_job_event_not_found(
+        self, admin_client: RestClient, job_event_update_request
+    ):
+        job_event_id = uuid4()
+        with pytest.raises(
+            JobEventUpdateError, match=r"Job event with ID .* not found"
+        ):
+            await admin_client.aupdate_job_event(job_event_id, job_event_update_request)
